﻿namespace Netmash.Security.Authentication.Tuitio.Constants
{
    /// <summary>
    /// All claims except "IsAnonymousGuest" are calculated based on the roles the user has.
    /// IsAnonymousGuest is calculated by the client application through the "AuthenticateAsGuest" option it sent to the authentication handler.
    /// More precisely, the handler checks the existence and validity of the token, and in case of a negative result, if there is a declared "AuthenticateAsGuest" function, it means that the client application also wants to be contacted before the authentication fails.
    /// This function receives as a parameter the entire http request and will have full power to decide whether to authorize it or not.
    /// If the client application decides to authorize the request based on its internal logic, a ticket will be created with three claims (GuestId, GuestName and IsAnonymousGuest: true).
    /// The id and name of the guest are also given in the configuration by the client application, similar to the "AuthenticateAsGuest" function. Their role is strictly to be used to build the ticket and to be made available to the client application afterwards.
    /// </summary>
    public struct ClaimTypes
    {
        public const string
            UserName = "UserName",
            FirstName = "FirstName",
            LastName = "LastName",
            UserGroups = "UserGroups",
            UserRoles = "UserRoles",
            IsAnonymousGuest = "IsAnonymousGuest";
    }

    public struct ClaimIssuer
    {
        public const string
            Tuitio = "Tuitio";
    }
}
