﻿using Microsoft.AspNetCore.Authentication;
using Microsoft.Extensions.Logging;
using Netmash.Security.Authentication.Tuitio.Models;
using System;
using System.Linq;
using System.Security.Claims;
using Tuitio.PublishedLanguage.Dto;
using c = Netmash.Security.Authentication.Tuitio.Constants;

namespace Netmash.Security.Authentication.Tuitio.Services
{
    internal class AuthenticationTicketService
    {
        private readonly string _schemeName;
        private readonly ILogger _logger;

        public AuthenticationTicketService(string schemeName, ILogger logger)
        {
            _schemeName = schemeName ?? throw new ArgumentNullException(nameof(schemeName));
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
        }

        public AuthenticationTicket GetGuestAuthenticationTicket(int guestId, string guestName)
        {
            var claims = new[] {
                new Claim(ClaimTypes.NameIdentifier, guestId.ToString()),
                new Claim(ClaimTypes.Name, guestName),
                new Claim(c.ClaimTypes.IsAnonymousGuest, bool.TrueString)
            };

            var ticket = GetAuthenticationTicket(claims);
            return ticket;
        }

        public AuthenticationTicket GetAuthenticationTicket(AuthorizationResult authorization)
        {
            if (authorization == null)
                throw new ArgumentNullException(nameof(authorization));

            var claimCollection = new ClaimCollection()
            {
                { ClaimTypes.NameIdentifier, authorization.UserId },
                { ClaimTypes.Name, authorization.UserName },
                { c.ClaimTypes.UserName, authorization.UserName }
            };

            claimCollection.TryAddClaim(authorization.FirstName, ClaimTypes.GivenName, c.ClaimTypes.FirstName);
            claimCollection.TryAddClaim(authorization.LastName, ClaimTypes.Surname, c.ClaimTypes.LastName);
            claimCollection.TryAddClaim(authorization.Email, ClaimTypes.Email);
            claimCollection.TryAddClaim(authorization.UserGroups, c.ClaimTypes.UserGroups);
            claimCollection.TryAddClaim(authorization.UserRoles, c.ClaimTypes.UserRoles);

            claimCollection.TryAddRange(authorization.Claims, (key, value) =>
            {
                _logger.LogWarning($"There is already a claim with key {key} in the collection. The combination {key}:{value} will be ignored.");
            });

            var claims = claimCollection.Select(z => new Claim(z.Key, z.Value.ToString(), z.Value.GetType().Name, c.ClaimIssuer.Tuitio)).ToArray();
            var ticket = GetAuthenticationTicket(claims);

            return ticket;
        }

        private AuthenticationTicket GetAuthenticationTicket(Claim[] claims)
        {
            var identity = new ClaimsIdentity(claims, _schemeName);
            var principal = new ClaimsPrincipal(identity);
            var ticket = new AuthenticationTicket(principal, _schemeName);
            return ticket;
        }
    }
}
